@@ -5,7 +5,7 @@ import pytest
 from sphinx.errors import ConfigError, ExtensionError
 import sphinx_gallery
 from sphinx_gallery.gen_gallery import _complete_gallery_conf
-from sphinx_gallery.scrapers import (figure_rst, mayavi_scraper, SINGLE_IMAGE,
+from sphinx_gallery.scrapers import (figure_rst, mayavi_scraper, SG_IMAGE,
                                      matplotlib_scraper, ImagePathIterator,
                                      save_figures, _KNOWN_IMG_EXTS)
 from sphinx_gallery.utils import _get_image
@@ -32,7 +32,7 @@ class matplotlib_svg_scraper():
 
 
 @pytest.mark.parametrize('ext', ('png', 'svg'))
-def test_save_matplotlib_figures(gallery_conf, ext, req_mpl, req_pil):
+def test_save_matplotlib_figures(gallery_conf, ext):
     """Test matplotlib figure save."""
     if ext == 'svg':
         gallery_conf['image_scrapers'] = (matplotlib_svg_scraper(),)
@@ -64,6 +64,50 @@ def test_save_matplotlib_figures(gallery_conf, ext, req_mpl, req_pil):
         assert os.path.isfile(fname)
 
 
+def test_save_matplotlib_figures_hidpi(gallery_conf):
+    """Test matplotlib hidpi figure save."""
+    ext = 'png'
+    gallery_conf['image_srcset'] = ["2x"]
+
+    import matplotlib.pyplot as plt  # nest these so that Agg can be set
+    plt.plot(1, 1)
+    fname_template = os.path.join(gallery_conf['gallery_dir'], 'image{0}.png')
+    image_path_iterator = ImagePathIterator(fname_template)
+    block = ('',) * 3
+    block_vars = dict(image_path_iterator=image_path_iterator)
+    image_rst = save_figures(block, block_vars, gallery_conf)
+
+    fname = f'/image1.{ext}'
+    assert fname in image_rst
+    assert f'/image1_2_0x.{ext} 2.0x' in image_rst
+
+    assert len(image_path_iterator) == 1
+    fname = gallery_conf['gallery_dir'] + fname
+    fnamehi = gallery_conf['gallery_dir'] + f'/image1_2_0x.{ext}'
+
+    assert os.path.isfile(fname)
+    assert os.path.isfile(fnamehi)
+
+    # Test capturing 2 images with shifted start number
+    image_path_iterator.next()
+    image_path_iterator.next()
+    plt.plot(1, 1)
+    plt.figure()
+    plt.plot(1, 1)
+    image_rst = save_figures(block, block_vars, gallery_conf)
+    assert len(image_path_iterator) == 5
+    for ii in range(4, 6):
+        fname = f'/image{ii}.{ext}'
+        assert fname in image_rst
+
+        fname = gallery_conf['gallery_dir'] + fname
+        assert os.path.isfile(fname)
+        fname = f'/image{ii}_2_0x.{ext}'
+        assert fname in image_rst
+        fname = gallery_conf['gallery_dir'] + fname
+        assert os.path.isfile(fname)
+
+
 def test_save_mayavi_figures(gallery_conf, req_mpl, req_pil):
     """Test file naming when saving figures. Requires mayavi."""
     import numpy as np
@@ -178,38 +222,87 @@ def test_figure_rst(ext):
     """Testing rst of images"""
     figure_list = ['sphx_glr_plot_1.' + ext]
     image_rst = figure_rst(figure_list, '.')
-    single_image = """
-.. image:: /sphx_glr_plot_1.{ext}
-    :alt: pl
-    :class: sphx-glr-single-img
-""".format(ext=ext)
+    single_image = f"""
+.. image-sg:: /sphx_glr_plot_1.{ext}
+   :alt: pl
+   :srcset: /sphx_glr_plot_1.{ext}
+   :class: sphx-glr-single-img
+"""
     assert image_rst == single_image
 
     image_rst = figure_rst(figure_list + ['second.' + ext], '.')
 
-    image_list_rst = """
+    image_list_rst = f"""
 .. rst-class:: sphx-glr-horizontal
 
 
     *
 
-      .. image:: /sphx_glr_plot_1.{ext}
+      .. image-sg:: /sphx_glr_plot_1.{ext}
           :alt: pl
+          :srcset: /sphx_glr_plot_1.{ext}
           :class: sphx-glr-multi-img
 
     *
 
-      .. image:: /second.{ext}
+      .. image-sg:: /second.{ext}
           :alt: pl
+          :srcset: /second.{ext}
           :class: sphx-glr-multi-img
-""".format(ext=ext)
+"""
     assert image_rst == image_list_rst
 
     # test issue #229
     local_img = [os.path.join(os.getcwd(), 'third.' + ext)]
     image_rst = figure_rst(local_img, '.')
 
-    single_image = SINGLE_IMAGE % ("third." + ext, '')
+    single_image = SG_IMAGE % ("third." + ext, '', "/third." + ext)
+    assert image_rst == single_image
+
+
+@pytest.mark.parametrize('ext', ['png'])
+def test_figure_rst_srcset(ext):
+    """Testing rst of images"""
+    figure_list = ['sphx_glr_plot_1.' + ext]
+    hipaths = [{0: 'sphx_glr_plot_1.png', 2.0: 'sphx_glr_plot_1_2_0.png'}]
+    image_rst = figure_rst(figure_list, '.', srcsetpaths=hipaths)
+    single_image = f"""
+.. image-sg:: /sphx_glr_plot_1.{ext}
+   :alt: pl
+   :srcset: /sphx_glr_plot_1.{ext}, /sphx_glr_plot_1_2_0.{ext} 2.0x
+   :class: sphx-glr-single-img
+"""
+    assert image_rst == single_image
+
+    hipaths += [{0: 'second.png', 2.0: 'second_2_0.png'}]
+    image_rst = figure_rst(figure_list + ['second.' + ext], '.',
+                           srcsetpaths=hipaths+[])
+
+    image_list_rst = f"""
+.. rst-class:: sphx-glr-horizontal
+
+
+    *
+
+      .. image-sg:: /sphx_glr_plot_1.{ext}
+          :alt: pl
+          :srcset: /sphx_glr_plot_1.png, /sphx_glr_plot_1_2_0.png 2.0x
+          :class: sphx-glr-multi-img
+
+    *
+
+      .. image-sg:: /second.{ext}
+          :alt: pl
+          :srcset: /second.{ext}, /second_2_0.{ext} 2.0x
+          :class: sphx-glr-multi-img
+"""
+    assert image_rst == image_list_rst
+
+    # test issue #229
+    local_img = [os.path.join(os.getcwd(), 'third.' + ext)]
+    image_rst = figure_rst(local_img, '.')
+
+    single_image = SG_IMAGE % ("third." + ext, '', "/third." + ext)
     assert image_rst == single_image
 
 
